
/**
 * @file TM_motorDriver.cpp
 * 
 * @brief This is the motor driver of the ThinkingSkins - MossSolar Facade project
 *
 * @author Cristin Schroeder, Clemens Röhl, Georg Röhl & Maximilian Rein
 *
 **/


#include "TM_motorDriver.h"


TM_motorDriver::TM_motorDriver() {};

int stepsA = 0;  //represents the count of steps remaining for motor A
int stepsB = 0;  //represents the count of steps remaining for motor B
int stepsC = 0;  //represents the count of steps remaining for motor C

boolean directionA = false;  //represent the current direction of rotation for panel A
boolean directionB = false;  //represent the current direction of rotation for panel B
boolean directionC = false;  //represent the current direction of rotation for panel C

char sideA = 's';  //represent the current frontside of panel A
char sideB = 's';  //represent the current frontside of panel B
char sideC = 's';  //represent the current frontside of panel C

char turnBuffer[4] = "sss";
boolean turnsBuffered = false;

boolean turning = false;  //true while ongoing turning-actions

boolean calibrating = false;  //true while a calibration-process


/**
setup()
   @brief Initializes the pins for the motordrivers and set them to a defined state.
    Please call calibrateMotors() after this function to provide correct functionality
*/
void TM_motorDriver::setup() {
  // setup motor A
  pinMode(STEP_PIN_A, OUTPUT); 
  digitalWrite(STEP_PIN_A, LOW);
  pinMode(DIR_PIN_A, OUTPUT);
  digitalWrite(DIR_PIN_A, LOW);
  
  // setup motor B
  pinMode(STEP_PIN_B, OUTPUT); 
  digitalWrite(STEP_PIN_B, LOW);
  pinMode(DIR_PIN_B, OUTPUT);
  digitalWrite(DIR_PIN_B, LOW);
  
  // setup motor C
  pinMode(STEP_PIN_C, OUTPUT); 
  digitalWrite(STEP_PIN_C, LOW);
  pinMode(DIR_PIN_C, OUTPUT);
  digitalWrite(DIR_PIN_C, LOW);
  
  //setup calibration-pins
  pinMode(CAL_PIN_A, INPUT);
  digitalWrite(CAL_PIN_A, HIGH);
  pinMode(CAL_PIN_B, INPUT);
  digitalWrite(CAL_PIN_B, HIGH);
  pinMode(CAL_PIN_C, INPUT);
  digitalWrite(CAL_PIN_C, HIGH);
}


/**
loop()
   @brief heck each loop-tick if a motor has to be turned.
*/
void TM_motorDriver::loop() {
  //setup for parallel turning-actions 
  if(stepsA > 0) {
    turnMotorA();
    stepsA--;
  } 
  if(stepsB > 0) {
    turnMotorB();
    stepsB--;
  }
  if(stepsC > 0) {
    turnMotorC();
    stepsC--;
  } 
  
  // set boolean turning to check for ongoing turning-actions to avoid positioning errors
  if(!turning && (stepsA + stepsB + stepsC > 0)) {
    turning = true;
  }
  if(turning && (stepsA + stepsB + stepsC == 0)) {
    turning = false;
  }
  
  // check for execution of remaining turn-order in buffer
  if(turnsBuffered && !turning) {
    loadTurns();
    turning = true;
    turnsBuffered = false;  
  } 
}


/**
turnMotorA()
   @brief Turns the motor of panel A with one step.
    Called in loop() for parallel turning-actions.
*/
void TM_motorDriver::turnMotorA(){
  digitalWrite(DIR_PIN_A, directionA);
  delay(2);
  digitalWrite(STEP_PIN_A, HIGH);
  delay(2);  
  digitalWrite(STEP_PIN_A, LOW);
} 


/**
turnMotorB()
   @brief Turns the motor of panel B with one step.
    Called in loop() for parallel turning-actions.
*/
void TM_motorDriver::turnMotorB(){
  digitalWrite(DIR_PIN_B, directionB);
  delay(2);
  digitalWrite(STEP_PIN_B, HIGH);
  delay(2);  
  digitalWrite(STEP_PIN_B, LOW);
}


/**
turnMotorC()
  @brief Turns the motor of panel C with one step.
    Called in loop() for parallel turning-actions.
*/
void TM_motorDriver::turnMotorC(){
  digitalWrite(DIR_PIN_C, directionC);
  delay(2);
  digitalWrite(STEP_PIN_C, HIGH);
  delay(2);  
  digitalWrite(STEP_PIN_C, LOW);
}


/**
setPanels(byte sideA, byte sideB, byte sideC)
  @param psideA - set front of panel A
  @param psideB - set front of panel B
  @param psideC - set front of panel C
  
      set 'g' for green moss
      set 'r' for red moss
      set 's' for solar 
      
  @brief Turns the choosen layer of the panels to the frondside.
  
    If the layer is already at the frontside, nothing is done.
    If there is an ongoing turning-action, it buffers the next turning-order
    to execute it after the current action.
     
*/
void TM_motorDriver::setPanels(char pSideA, char pSideB, char pSideC) {
  if (!turning) {
    setMotorA(pSideA);
    setMotorB(pSideB);
    setMotorC(pSideC);
  } else {
    saveTurns(pSideA, pSideB, pSideC);  
    turnsBuffered = true;
  }
}


/**
saveTurns(char pSideA, char pSideB, char pSideC)
  @param pSideA - set front of panel A
  @param pSideB - set front of panel B
  @param pSideC - set front of panel C
  
  @brief Buffers a combination of frontsides for the three panels A,B,C.
*/
void TM_motorDriver::saveTurns(char pSideA, char pSideB, char pSideC) {
  turnBuffer[0] = pSideA;
  turnBuffer[1] = pSideB;
  turnBuffer[2] = pSideC;
}


/**
loadTurns()
  @brief Execute a previously blocked turning-order with the buffered combination.
*/
void TM_motorDriver::loadTurns() {
  setPanels(turnBuffer[0], turnBuffer[1], turnBuffer[2]); 
}


/**
calibrate()
  @brief Syncronize all panels to the neutral setting.
  
    No ongoing or upcoming turnings, left rotation-direction.
*/
void TM_motorDriver::calibrate() {
  //reset the settings
  directionA = false;
  directionB = false;
  directionC = false;
  stepsA = 0;
  stepsB = 0;
  stepsC = 0;
  turning = false;
  turnsBuffered = false;
  turnBuffer[0] = 's';
  turnBuffer[1] = 's';
  turnBuffer[2] = 's';
  
  //calibrating A
  int i = 0;
  while(digitalRead(CAL_PIN_A)) {
    turnMotorA(); 
  }
  delay(100);
  
  //calibrating B
  while(digitalRead(CAL_PIN_B)) {
    turnMotorB();   
  }
  delay(100);
  
  //calibrating C
  while(digitalRead(CAL_PIN_C)) {
    turnMotorC();   
  }  
}


void TM_motorDriver::setMotorA(char pSide) {
  Serial.println("setmotor check");
  switch(pSide) {
    case 'g':
      switch(sideA) {
        case 'r':
          directionA = true;
          stepsA = TWO_SIDE_TURNS;
        break;
        case 's':
          directionA = true;
          stepsA = ONE_SIDE_TURN;
        break;
      }
    break;
    case 'r':
      switch(sideA) {
        case 'g':
          directionA = false;
          stepsA = TWO_SIDE_TURNS;
        break;
        case 's':
          directionA = false;
          stepsA = ONE_SIDE_TURN;
          Serial.println("stepcount check");
        break;
      }
    break;
    case 's':
      switch(sideA) {
        case 'g':
          directionA = false;
          stepsA = ONE_SIDE_TURN;
        break;
        case 'r':
          directionA = true;
          stepsA = ONE_SIDE_TURN;
        break;
      }
    break;
  } 
}


void TM_motorDriver::setMotorB(char pSide) {
  switch(pSide) {
    case 'g':
      switch(sideB) {
        case 'r':
          directionB = true;
          stepsB = TWO_SIDE_TURNS;
        break;
        case 's':
          directionB = true;
          stepsB = ONE_SIDE_TURN;
        break;
      }
    break;
    case 'r':
      switch(sideB) {
        case 'g':
          directionB = false;
          stepsB = TWO_SIDE_TURNS;
        break;
        case 's':
          directionB = false;
          stepsB = ONE_SIDE_TURN;
        break;
      }
    break;
    case 's':
      switch(sideB) {
        case 'g':
          directionB = false;
          stepsB = ONE_SIDE_TURN;
        break;
        case 'r':
          directionB = true;
          stepsB = ONE_SIDE_TURN;
        break;
      }
    break;
  } 
}


void TM_motorDriver::setMotorC(char pSide) {
  switch(pSide) {
    case 'g':
      switch(sideC) {
        case 'r':
          directionC = true;
          stepsC = TWO_SIDE_TURNS;
        break;
        case 's':
          directionC = true;
          stepsC = ONE_SIDE_TURN;
        break;
      }
    break;
    case 'r':
      switch(sideC) {
        case 'g':
          directionC = false;
          stepsC = TWO_SIDE_TURNS;
        break;
        case 's':
          directionC = false;
          stepsC = ONE_SIDE_TURN;
        break;
      }
    break;
    case 's':
      switch(sideC) {
        case 'g':
          directionC = false;
          stepsC = ONE_SIDE_TURN;
        break;
        case 'r':
          directionC = true;
          stepsC = ONE_SIDE_TURN;
        break;
      }
    break;
  } 
}


