
/**
 * @file thinkingSkins_Moss.ino
 * 
 * @brief This is the mainfile of the ThinkingSkins - MossSolar Facade project software
 *
 * This code provides funcions to turn the three panels A,B,C with the sides 
 *   green, red and solar.
 * Change the triggerevent-functions below setup/loop for your personal needs
 * Call motors.calibrate() to calibrate the panel-alignment
 *
 * @author Cristin Schroeder, Clemens Röhl, Georg Röhl & Maximilian Rein
 *
 **/


//includes

//pins for sensors, motordrivers, led, thresholds for sensorvalue comparison
#include "defines.h"
#include "TM_motorDriver.h"

//lightSensor
#include <Adafruit_Sensor.h>
#include <Adafruit_TSL2561_U.h>
#include <Wire.h>

//RGB_LED
#include <Adafruit_NeoPixel.h>
#include <pgmspace.h>
#include "WS2812_Definitions.h"


unsigned long dustSensorTimer = 0;         // contains the time when the next dust-measurement is triggered 
unsigned long lightSensorTimer = 0;        // contains the time when the next light-measurement is triggered
unsigned long temperatureSensorTimer = 0;  // contains the time when the next temperature-measurement is triggered
unsigned long humiditySensorTimer = 0;     // contains the time when the next humidity-measurement is triggered

unsigned long time = 0;  // contains the runtime for the checks in the loop-function, set by millis()

int particles = 0;   // value from the dustsensor, shows the amount of particles in the air
int lux = 0;         // value from the lightsensor, shows the luminosity at the front 
int temperature = 0; // value from the temperaturesensor, shows the temperature 
int humidity_A = 0;  // value from the first humiditysensor (A), shows the humidity 
int humidity_B = 0;  // value from the second humiditysensor (B), shows the humidity 

boolean dustSensoredPrev = false;        // true if the first dust-measurement in a row is positive, else false 
boolean lightSensoredPrev = false;       // true if the first light-measurement in a row is positive, else false 
boolean temperatureSensoredPrev = false; // true if the first temperature-measurement in a row is positive, else false 
boolean humiditySensoredPrev_A = false;  // true if the first humidity-measurement of sensor A in a row is positive, else false 
boolean humiditySensoredPrev_B = false;  // true if the first humidity-measurement of sensor B in a row is positive, else false 

Adafruit_NeoPixel colorLed = Adafruit_NeoPixel(1, LED_PIN, NEO_GRB + NEO_KHZ800);            // class definition for the RGB-LED
Adafruit_TSL2561_Unified lightSensor = Adafruit_TSL2561_Unified(TSL2561_ADDR_FLOAT, 11235);  // class definition for the lightsensor

sensors_event_t lightEvent;  // triggerEvent for the lightsensor

TM_motorDriver motors = TM_motorDriver();  // interface for the motordriver


/*********************************
************  SETUP  *************
**********************************/
void setup() {
  motors.setup();  // initializes pins etc. for the motors and calibration
  setupSensors();  // initializes pins etc. for the sensors and set defined pinstates
}


/*********************************
*************  LOOP  *************
**********************************/
void loop() {   // a continuous loop (w/o blocking functions) is required to provide functionality
  time = millis();
  motors.loop();  
  loopDustSensor();
  loopLightSensor(); 
  loopTemperatureSensor();
  loopHumiditySensor();
}


/*********************************
********  TRIGGEREVENTS  *********
**********************************/

/******************************************************************************
* This functions are called if a sensorcheck
* detects the passing of its threshold.
* Adjust for your needs.
* Possible functions:
*   Call motors.setPanels('x', 'y', 'z') to turn the selected layer to the  
*     frontside (replays x,y,z with the chars g (green), r (red) or s (solar).
*     If there is an ongoing turning-action, the last called order with this
*     function will automatically be execute after this action.
*   Call testXYZSensor() to get a comparison-boolean of the sensorvalue
*     and the threshold. For details of the funcions, look below.
*   Call colorLed.setPixelColor(0, ABC) to set the color of the RGB-LED.
*     To lighten up the led with the new color, call colorLed.show().
*     Open WS2812_Definitions.h for color-code information.
*   Call motors.calibrate() to calibrate the panel-alignment
********************************************************************************/
/**
triggerDustEvent()
  @brief Called, if there is much dust in the air.
    Randomly turns one of the moss-sides to the front
*/
void triggerDustEvent() {
  if (random(2)) {
    motors.setPanels('g','g','g');
  } else {
    motors.setPanels('r','r','r');
  }
  colorLed.setPixelColor(0, GREEN);
  colorLed.show();
}


/**
triggerLightEvent()
  @brief Called, if the brightness at the frondside is to high.
    Turns the solarpanels to the front to protect the moss
    and collect energy.
*/
void triggerLightEvent() {
  motors.setPanels('s','s','s');
  colorLed.setPixelColor(0, YELLOW);
  colorLed.show();
}


/**
triggerTemperatureEvent()
   @brief  Called, if the temperature at the frondside is to high.
    Turns the solarpanels to the front to protect the moss
      and switchs over the status-led to red to indicate the
      watering prohibition .
*/
void triggerTemperatureEvent() {
  colorLed.setPixelColor(0, RED);
  colorLed.show();
}


/**
triggerHumidityEvent_A()
   @brief Called, if the A-side of the moss (green or 'g') runs dry.
    Turn the other moss-side to the front ('r') and switchs over
      the status-led to blue to indicate the watering requirement.
*/
void triggerHumidityEvent_A() {
  motors.setPanels('r','r','r');
  colorLed.setPixelColor(0, BLUE);
  colorLed.show();
}



/**
triggerHumidityEvent_B()
   @brief Called, if the B-side of the moss (red or 'r') runs dry.
    Turn the other moss-side to the front ('g') and switchs over
      the status-led to blue to indicate the watering requirement.
*/
void triggerHumidityEvent_B() {
  motors.setPanels('g','g','g');
  colorLed.setPixelColor(0, BLUE);
  colorLed.show();
}




/**
setupSensors()
   @brief Initializes the pins for the sensors and the led and set them to a defined state.
*/
void setupSensors() {
  colorLed.begin(); 
  colorLed.setPixelColor(1, 0); 
  colorLed.show(); 

  while (!lightSensor.begin());
  lightSensor.enableAutoRange(true);
  lightSensor.setIntegrationTime(TSL2561_INTEGRATIONTIME_101MS);


  pinMode(DUSTSENSOR_SET_PIN, OUTPUT);
}


/**
loopDustSensor()
   @brief Checks each loop-tick if the dustsensor-timer triggers a check.
   
    Standartinterval: 10s
    Debounceinterval after first trigger: 5s
    Interval after a positive triggered dustsensoring: 20s
    After dust is sensored the first time, the sensor waits for 5s and check,
      whether there is still dust. Turns the moss in front if this check is also positive.
 */
void loopDustSensor() {
  if(dustSensorTimer < time) {
    if(testDustSensor()) {
      if(dustSensoredPrev) {
        triggerDustEvent(); 
        dustSensoredPrev = false;
        dustSensorTimer = millis() + DUSTSENSOR_TRIGGERED_CHECKTIME;
      } 
      else {
        dustSensoredPrev = true;
        dustSensorTimer = millis() + DUSTSENSOR_DEBOUNCE;
      }
    } 
    else {
      dustSensorTimer = millis() + DUSTSENSOR_IDLE_CHECKTIME;
    }
  }  
}


/**
loopLightSensor()
   @brief Checks each loop-tick if the lightsensor-timer triggers a check.
   
    Standartinterval: 10s
    Debounceinterval after first trigger: 5s
    Interval after a positive triggered lightsensoring: 20s
    After light is sensored the first time, the sensor waits for 5s and check,
      whether its still too bright. Turns the solarpanels in front if this check is also positive.
 */
void loopLightSensor() {
  if(lightSensorTimer < time) {
    if(testLightSensor()) {
      if(lightSensoredPrev) {
        triggerLightEvent();
        lightSensorTimer = millis() + LIGHTSENSOR_TRIGGERED_CHECKTIME;
      } 
      else {
        lightSensoredPrev = true;
        lightSensorTimer = millis() + LIGHTSENSOR_DEBOUNCE;
      }
    } 
    else {
      lightSensorTimer = millis() + LIGHTSENSOR_IDLE_CHECKTIME;
    }
  }  
}


/**
loopTemperatureSensor()
   @brief Checks each loop-tick if the temperaturesensor-timer triggers a check.
   
   Standartinterval: 10s
   Debounceinterval after first trigger: 5s
   Interval after a positive triggered temperaturesensoring: 20s
   After a high temperature is sensored the first time, the sensor waits for 5s and check,
     whether its still too hot. Turns the solarpanels in front if this check is also positive.
 */
void loopTemperatureSensor() {
  if(temperatureSensorTimer < time) {
    if(testTemperatureSensor()) {
      if(temperatureSensoredPrev) {
        triggerTemperatureEvent();
        temperatureSensorTimer = millis() + TEMPERATURESENSOR_TRIGGERED_CHECKTIME;
      } 
      else {
        temperatureSensoredPrev = true;
        temperatureSensorTimer = millis() + TEMPERATURESENSOR_DEBOUNCE;
      }
    } 
    else {
      temperatureSensorTimer = millis() + TEMPERATURESENSOR_IDLE_CHECKTIME;
    }
  }  
}


/**
loopHumiditySensor()
   @brief Checks each loop-tick if the humitidysensor-timer triggers a check.
   
    Standartinterval: 60s
    Debounceinterval after first trigger: 5s
    Interval after a positive triggered humiditysensoring: 20s
    After drought is sensored the first time in one og the mosssides, the sensor waits for 5s and check,
      whether the moss is still dry. Turns the solarpanels in front if this check is also positive.
 */
void loopHumiditySensor() {
  if(humiditySensorTimer < time) {
    if(testHumiditySensor_A() || testHumiditySensor_B()) {
      if(testHumiditySensor_A()) {            //either A is dried out 
        if(humiditySensoredPrev_A) {
          triggerHumidityEvent_A();
          humiditySensorTimer = millis() + HUMIDITYSENSOR_TRIGGERED_CHECKTIME;
        } 
        else {
          humiditySensoredPrev_A = true;
          humiditySensorTimer = millis() + HUMIDITYSENSOR_DEBOUNCE;
        }
      } 
      else {                                   //or B
        if(humiditySensoredPrev_B) {
          triggerHumidityEvent_B();
          humiditySensorTimer = millis() + HUMIDITYSENSOR_TRIGGERED_CHECKTIME;
        } 
        else {
          humiditySensoredPrev_B = true;
          humiditySensorTimer = millis() + HUMIDITYSENSOR_DEBOUNCE;
        }  
      }      
    } 
    else {
      humiditySensorTimer = millis() + HUMIDITYSENSOR_IDLE_CHECKTIME;
    }
  }  
}


/**
readDustSensor()
  Returns:
    int which contains the value of dustparticles messured in the air.
      Higher value => more dustparticles
      0 => zero dustparticles (theoretical)
      
  @brief First lighten the led in the dustsensor. Then wait and
         then get the value from the internal lightsensor and return it.
 */
int readDustSensor(){
  int inputValue = 0;
  digitalWrite(DUSTSENSOR_SET_PIN,LOW);
  delayMicroseconds(EXPOSURE_TIME);
  inputValue = analogRead(DUSTSENSOR_READ_PIN);
  delayMicroseconds(ACQUISITION_TIME);
  digitalWrite(DUSTSENSOR_SET_PIN,HIGH);
  return inputValue; 
}


/**
testDustSensor()
  Return:
    true, if the measured value passes the threshold
    false, if it is within the threshold
    
  @brief Reads the value of the dustsensor and compares it with the threshold.
*/
boolean testDustSensor() {
  particles = readDustSensor();
  return (particles > DUST_THRESHOLD);
}


/**
testLightSensor()
  Return:
    true, if the measured value passes the threshold
    false, if it is within the threshold
    
  @brief eads the value of the lightsensor and compares it with the threshold.
*/
boolean testLightSensor() {
  lightSensor.getEvent(&lightEvent);
  lux = lightEvent.light;
  return (lux > LUX_THRESHOLD);
}


/**
testTemperatureSensor()
  Return:
    true, if the measured value passes the threshold
    false, if it is within the threshold
    
  @brief Reads the value of the temperaturesensor and compares it with the threshold.
*/
boolean testTemperatureSensor() {
  temperature = analogRead(TEMPERATURESENSOR_PIN);
  return (temperature > TEMPERATURE_THRESHOLD);
}


/**
testHumiditySensor_A()
  Return:
    true, if the measured value passes the threshold
    false, if it is within the threshold
    
  @brief Reads the value of the first humiditysensor (A) and compares it with the threshold.
*/
boolean testHumiditySensor_A() {
  humidity_A = analogRead(HUMIDITYSENSOR_A_PIN);
  return (humidity_A < HUMIDITY_THRESHOLD);
}


/**
testHumiditySensor_B()
  Return:
    true, if the measured value passes the threshold
    false, if it is within the threshold
    
  @brief Reads the value of the second humiditysensor (B) and compares it with the threshold.
*/
boolean testHumiditySensor_B() {
  humidity_B = analogRead(HUMIDITYSENSOR_B_PIN);
  return (humidity_B < HUMIDITY_THRESHOLD);
}




